// img_vc++.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "img_vc++.h"
#include <gdiplus.h>
using namespace Gdiplus;
#pragma comment(lib, "gdiplus.lib")




#ifdef _DEBUG
#define new DEBUG_NEW
#endif






// The one and only application object

CWinApp theApp;

using namespace std;


///-----------------------------------------------------------------
///
///-----------------------------------------------------------------
int GetCodecClsid(const WCHAR* format, CLSID* pClsid)
{
	UINT num = 0;  // number of image encoders
	UINT size = 0; // size of the image encoder array in bytes

	ImageCodecInfo* pImageCodecInfo = NULL;

	GetImageEncodersSize(&num, &size);
	if(size == 0)
	return -1; // Failure

	pImageCodecInfo = (ImageCodecInfo*)(malloc(size));
	if(pImageCodecInfo == NULL)
		return -1; // Failure

	GetImageEncoders(num, size, pImageCodecInfo);

	for(UINT j = 0; j < num; ++j)
	{
		if( wcscmp(pImageCodecInfo[j].MimeType, format) == 0 )
		{
			*pClsid = pImageCodecInfo[j].Clsid;
			return j; // Success
		}
	} // for

	return -1; // Failure
} // GetCodecClsid





///-----------------------------------------------------------------
///
///-----------------------------------------------------------------
void DumpRgb(TCHAR* imgSrc, TCHAR* imgTarget)
{
    /// 2006-06-22 brucewang
    /// Fixed FILE LOCK Problem.
    //Bitmap *fileImage = fileImage.FromFile(imgSrc) as Bitmap;
	Bitmap *MainImage = Bitmap::FromFile(imgSrc);
	if(MainImage)
	{
		BitmapData bmpdata;
		MainImage->LockBits(
			new Rect(0, 0, MainImage->GetWidth(), MainImage->GetHeight() ),
			ImageLockMode::ImageLockModeRead | ImageLockMode::ImageLockModeWrite,
			PixelFormat24bppRGB,
			&bmpdata);

		FILE *fp = _tfopen( imgTarget, _T("wb")); //(TCHAR*)"dump", _T("wb") );//

		int stride = bmpdata.Stride;
		void* Scan0 = bmpdata.Scan0;

		byte* p = (byte*)(void*)Scan0;
		int nOffset = stride - MainImage->GetWidth() * 3;
		unsigned char red, green, blue;
		for (int y = 0; y < MainImage->GetHeight() ; ++y)
		{
			for (int x = 0; x < MainImage->GetWidth(); ++x)
			{
				fwrite(p, 1, 3, fp);
				p += 3;
			}
			p += nOffset;
		}

		fclose(fp);
	}
}





///-----------------------------------------------------------------
///
///-----------------------------------------------------------------
void GreyscaleImage(TCHAR* imgSrc, TCHAR* imgTarget)
{
    /// 2006-06-22 brucewang
    /// Fixed FILE LOCK Problem.
    //Bitmap *fileImage = fileImage.FromFile(imgSrc) as Bitmap;
	Bitmap *MainImage = Bitmap::FromFile(imgSrc);
	if(MainImage)
	{
		BitmapData bmpdata;
		MainImage->LockBits(
			new Rect(0, 0, MainImage->GetWidth(), MainImage->GetHeight() ),
			ImageLockMode::ImageLockModeRead | ImageLockMode::ImageLockModeWrite,
			PixelFormat24bppRGB,
			&bmpdata);

		int stride = bmpdata.Stride;
		void* Scan0 = bmpdata.Scan0;

		byte* p = (byte*)(void*)Scan0;
		int nOffset = stride - MainImage->GetWidth() * 3;
		unsigned char red, green, blue;
		for (int y = 0; y < MainImage->GetHeight() ; ++y)
		{
			for (int x = 0; x < MainImage->GetWidth(); ++x)
			{
				blue = p[0];
				green = p[1];
				red = p[2];

				p[0] = p[1] = p[2] = (unsigned char)(.299 * red + .587 * green + .114 * blue);
				p += 3;
			}
			p += nOffset;
		}


		MainImage->UnlockBits(&bmpdata);

		CLSID PngCodec;
		GetCodecClsid(L"image/png", &PngCodec);
		MainImage->Save(imgTarget, &PngCodec, NULL);
	}
}



///-----------------------------------------------------------------
///
///-----------------------------------------------------------------
void EncryptImage(TCHAR* imgSrc, TCHAR* imgTarget, TCHAR* encKeyString)
{
    Bitmap *MainImage = Bitmap::FromFile(imgSrc);
	if(MainImage)
	{
		// Lock the bitmap's bits.  
		BitmapData bmpData;
		MainImage->LockBits(
			new Rect(0, 0, MainImage->GetWidth(), MainImage->GetHeight() ),
			ImageLockMode::ImageLockModeRead | ImageLockMode::ImageLockModeWrite,
			PixelFormat24bppRGB,
			&bmpData);

		

		int keylength = _tcslen(encKeyString);//encKeyString.GetLength();
		unsigned char* enckey = new unsigned char[keylength];

		for (int i = 0; i < keylength; i++)
		{
			enckey[i] = (unsigned char)encKeyString[i];
		}

		int count = 0;
		int stride = bmpData.Stride;
		void* Scan0 = bmpData.Scan0;

		unsigned char* p = (unsigned char*)(void*)Scan0;
		int nOffset = stride - MainImage->GetWidth() * 3;
		int nWidth = MainImage->GetWidth() * 3;

		int ikeyiteration = 0; //!!
		for (int y = 0; y < MainImage->GetHeight(); ++y)
		{
			for (int x = 0; x < nWidth; ++x)
			{
				p[0] ^= enckey[ikeyiteration];
				enckey[ikeyiteration] = (byte)(p[0]);

				++p;
				ikeyiteration++; //!!
				if (ikeyiteration >= keylength) ikeyiteration = 0;
			}
			p += nOffset;
		}


		MainImage->UnlockBits(&bmpData);
		CLSID PngCodec;
		GetCodecClsid(L"image/png", &PngCodec);
		MainImage->Save(imgTarget, &PngCodec, NULL);
	}
}




///-----------------------------------------------------------------
///
///-----------------------------------------------------------------
void ResizeImage(TCHAR* imgSrc, TCHAR* imgTarget, int NewWidth, int NewHeight)
{
    int width = 0, height = 0;

    /// 2006-06-22 brucewang
    Bitmap *MainImage = Bitmap::FromFile(imgSrc);
	if(MainImage)
	{
		// create a temporary Bitmap
		Bitmap bit(NewWidth, NewHeight, PixelFormat24bppRGB);
		// create its clone for returning
		Bitmap *smallerImg = bit.Clone(0, 0, bit.GetWidth(), bit.GetHeight(),PixelFormat24bppRGB);
		Graphics *g = Graphics::FromImage(smallerImg);
		//
		g->SetSmoothingMode(SmoothingMode::SmoothingModeHighQuality);
		g->SetInterpolationMode(InterpolationMode::InterpolationModeHighQualityBicubic);
		g->SetPixelOffsetMode(PixelOffsetMode::PixelOffsetModeHighQuality);


		/*
		//
		// Copy all image property of MainImage to small image
		//
		UINT totalBufferSize;
		UINT numProperties;
		MainImage->GetPropertySize(&totalBufferSize, &numProperties);
		// Allocate the buffer that will receive the property items.
		PropertyItem* pAllItems = (PropertyItem*)malloc(totalBufferSize);
		// Fill the buffer.
		MainImage->GetAllPropertyItems(totalBufferSize, numProperties, pAllItems);
		// Print the id data member of each property item.
		for(UINT j = 0; j < numProperties; ++j)
		{
			smallerImg->SetPropertyItem( pAllItems +j );
		}
		free(pAllItems);
		*/

		//
		//
		//
		if (MainImage->GetWidth() >= NewWidth || MainImage->GetHeight() >= NewHeight)
		{
			height = (int)(MainImage->GetHeight() * NewWidth / MainImage->GetWidth() );
			width = (int)(MainImage->GetWidth() * NewHeight / MainImage->GetHeight() );
			if (height > NewHeight) height = NewHeight;
			if (width > NewWidth) width = NewWidth;

		}
		else
		{
			width = MainImage->GetWidth();
			height = MainImage->GetHeight();
		}
		
		
		//
		//
		//
		g->DrawImage(
			MainImage,
			(NewWidth - width) / 2,
			(NewHeight - height) / 2,
			width,
			height);
		//
		//
		//
		CLSID PngCodec;
		GetCodecClsid(L"image/png", &PngCodec);
		smallerImg->Save(imgTarget, &PngCodec, NULL);
		delete smallerImg;
	}
}



///-----------------------------------------------------------------
///
///-----------------------------------------------------------------
void GenSignatureFromImage(TCHAR* imgSrc, TCHAR* imgTarget)
{
	TCHAR filetmp[MAX_PATH]={0,};
	_tcscpy(filetmp,imgSrc);
	_tcscat(filetmp,_T("temp.png"));

    GreyscaleImage(imgSrc, filetmp);
    ResizeImage(filetmp, imgTarget, 130, 90);
	::DeleteFile(filetmp);
}



///-----------------------------------------------------------------
///                     M A I N
///-----------------------------------------------------------------
int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	int nRetCode = 0;

	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// TODO: change error code to suit your needs
		_tprintf(_T("Fatal Error: MFC initialization failed\n"));
		nRetCode = 1;
	}
	else
	{
		TCHAR* operationMode;

		if (argc > 1)
        {
            operationMode = argv[1];
        }
        else
        {
            printf("Usage : img.exe [mode] [srcimg] [targetimg]\n");
            printf("    [mode] 'thumb': [srcimg] [targetimg] [width-pixel] [height-pixel]\n");
            printf("    [mode] 'encrypt': [srcimg] [targetimg] [key string]\n");
            printf("    [mode] 'grayscale': [srcimg] [targetimg]\n");
            printf("    [mode] 'gensig': [srcimg] [targetimg]\n");
			printf("    [mode] 'dump': [srcimg] [targetimg]\n");
            return 0;
        }



		//
		// TODO: code your application's behavior here.
		// Initialize GDI+
		//
		GdiplusStartupInput gdiplusStartupInput;
		unsigned long m_gdiplusToken;
		GdiplusStartup(&m_gdiplusToken, &gdiplusStartupInput, NULL);

		//
		//
		//
		if ( _tcscmp(operationMode, _T("thumb") ) == 0 )
        {
            int NewWidth = _ttoi( argv[4]);
			int NewHeight = _ttoi( argv[5]);
			ResizeImage(argv[2],argv[3],NewWidth,NewHeight);
        }
        else if ( _tcscmp(operationMode, _T("encrypt") ) == 0 )
        {
            EncryptImage(argv[2],argv[3],argv[4]);
        }
        else if ( _tcscmp(operationMode, _T("grayscale") ) == 0 )
        {
            GreyscaleImage(argv[2],argv[3]);
        }
        else if ( _tcscmp(operationMode, _T("gensig") ) == 0 )
        {
            GenSignatureFromImage(argv[2],argv[3]);
        }
		else if ( _tcscmp(operationMode, _T("dump") ) == 0 )
        {
			DumpRgb(argv[2],argv[3]);
        }
        else
        {
            printf("Unsupported command\n");
        }
		
		//
		
		//
		
		


		//
		//
		//
		GdiplusShutdown(m_gdiplusToken);
	}

	return nRetCode;
}
